'use strict';

var normalizeValidator = require('./normalize-validator');
var strictMode = require('./strict-mode');

/**
 * @param {String} functionName
 * @param {Function} functionTemplate
 * @param {NodePath} functionPath
 * @param {Object} jsDoc
 * @param {Boolean} useStrict
 * @param {Object} t
 */
module.exports = function (functionName, functionTemplate, functionPath, jsDoc, useStrict, t) {
    var parameters = jsDoc.parameters;
    var functionParameters = functionPath.get('params');
    var functionBody = functionPath.get('body');

    if (!functionBody.isBlockStatement()) return;

    var countOfInsertedArguments = insertAssertionBasedOn(functionParameters);

    if (useStrict) {
        var parametersKeys = Object.keys(parameters);

        if (parametersKeys.length > countOfInsertedArguments) {
            strictMode.throwException(functionPath, strictMode.ERROR.UNUSED_ARGUMENTS_IN_JSDOC);
        }
    }

    /**
     * @param {Array<NodePath>} nodes
     * @returns {Number}
     */
    function insertAssertionBasedOn(nodes) {
        var shouldThrowException = true;
        var count = 0;
        var path = void 0,
            node = void 0,
            name = void 0;
        var type = void 0;

        for (var index = nodes.length - 1; index >= 0; index--) {
            shouldThrowException = true;
            path = nodes[index];
            node = path.node;

            if (path.isAssignmentPattern()) {
                node = node.left;
            } else if (path.isRestElement()) {
                node = node.argument;
            } else if (path.isObjectProperty()) {
                shouldThrowException = false;
                count += insertAssertionBasedOn([path.get('value')]);
            } else if (path.isObjectPattern()) {
                shouldThrowException = false;
                count += insertAssertionBasedOn(path.get('properties'));
            } else if (path.isArrayPattern()) {
                shouldThrowException = false;
                count += insertAssertionBasedOn(path.get('elements'));
            }

            name = node.name;
            type = parameters[name];

            if (!(name in parameters) || type === void 0) {
                if (useStrict && shouldThrowException) {
                    strictMode.throwException(path, strictMode.ERROR.NO_ARGUMENT_IN_JSDOC);
                }

                continue;
            }

            ++count;

            if (!type) {
                continue;
            }

            functionBody.unshiftContainer('body', functionTemplate(functionName, name, node, normalizeValidator(functionPath, type, t)));
        }

        return count;
    }
};